import 'package:equatable/equatable.dart';
import 'package:syncrow_web/pages/device_managment/one_gang_switch/models/wall_light_status_model.dart';

class WallLightSwitchState extends Equatable {
  @override
  List<Object?> get props => [];
}

class WallLightSwitchInitial extends WallLightSwitchState {}

class WallLightSwitchLoading extends WallLightSwitchState {}

class WallLightSwitchStatusLoaded extends WallLightSwitchState {
  final WallLightStatusModel status;

  WallLightSwitchStatusLoaded(this.status);

  @override
  List<Object> get props => [status];
}

class WallLightSwitchError extends WallLightSwitchState {
  final String message;

  WallLightSwitchError(this.message);

  @override
  List<Object> get props => [message];
}

class WallLightSwitchControlError extends WallLightSwitchState {
  final String message;

  WallLightSwitchControlError(this.message);

  @override
  List<Object> get props => [message];
}

class WallLightSwitchBatchControlError extends WallLightSwitchState {
  final String message;

  WallLightSwitchBatchControlError(this.message);

  @override
  List<Object> get props => [message];
}

class WallLightSwitchBatchStatusLoaded extends WallLightSwitchState {
  final List<String> status;

  WallLightSwitchBatchStatusLoaded(this.status);

  @override
  List<Object> get props => [status];
}
